@@ -21,7 +21,7 @@ const_string_output = partial(const_string_output, cpl=True)
 double_output = partial(double_output, cpl=True)
 
 # Raster Driver Routines
-register_all = void_output(std_call('GDALAllRegister'), [])
+register_all = void_output(std_call('GDALAllRegister'), [], errcheck=False)
 get_driver = voidptr_output(std_call('GDALGetDriver'), [c_int])
 get_driver_by_name = voidptr_output(std_call('GDALGetDriverByName'), [c_char_p], errcheck=False)
 get_driver_count = int_output(std_call('GDALGetDriverCount'), [])
@@ -30,10 +30,7 @@ get_driver_description = const_string_output(std_call('GDALGetDescription'), [c_
 # Raster Data Source Routines
 create_ds = voidptr_output(std_call('GDALCreate'), [c_void_p, c_char_p, c_int, c_int, c_int, c_int, c_void_p])
 open_ds = voidptr_output(std_call('GDALOpen'), [c_char_p, c_int])
-if GDAL_VERSION >= (2, 0):
-    close_ds = voidptr_output(std_call('GDALClose'), [c_void_p])
-else:
-    close_ds = void_output(std_call('GDALClose'), [c_void_p])
+close_ds = void_output(std_call('GDALClose'), [c_void_p], errcheck=False)
 flush_ds = int_output(std_call('GDALFlushCache'), [c_void_p])
 copy_ds = voidptr_output(
     std_call('GDALCreateCopy'),
